;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-

;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          api.lisp
;;;; Purpose:       cl-digg Common Lisp API
;;;; Programmer:    Nicolas Lamirault <nicolas.lamirault@gmail.com>
;;;;
;;;; This file, part of cl-digg, is Copyright (c) 2007 by Nicolas Lamirault
;;;;
;;;; cl-digg are granted the rights to distribute and use this software
;;;; as governed by the terms of the Lisp Lesser GNU Public License
;;;; (http://opensource.franz.com/preamble.html), also known as the LLGPL.
;;;;
;;;; *************************************************************************


(in-package :cl-digg)


(defmacro with-digg-request ((request) choice &body body)
  "Macro with creates a HTTP request and executes BODY."
  `(let ((,request (concatenate 'string
                                *digg-server*
                                ,choice
                                "appkey="
                                *api-key*)))
     ,@body))



(defun extract-error (xml)
  "Extract code and message of Digg's error xml string.
Return code and message."
  (when *debug*
    (format t "Digg Error ~A~%" xml))
  (let* ((start-code (search "code=" xml))
         (end-code (search "message=" xml))
         (end-msg (search "/>" xml)))
    ;;(format t "Code: ~A Msg: ~A"
    (values (subseq xml (+ 6 start-code) (- end-code 2))
            (subseq xml (+ 9 end-code) (- end-msg 2)))))


(defun digg-perform (url)
  "Make a query to the Digg web service.
If HTTP return code is 200, returns the body-stream of the HTTP request.
Otherwise throws a DIGG-REQUEST-ERROR."
  (when *debug*
    (format t "~&Digg : ~A~%" url))
  (multiple-value-bind (body-stream status-code headers uri stream must-close)
      (drakma:http-request url)
    (declare (ignore headers uri stream must-close))
    (when *debug*
      (format t "Digg HTTP ~A." status-code))
    (if (and status-code (= status-code 200))
        body-stream
        (multiple-value-bind (code msg)
            (extract-error body-stream)
          (error 'digg-request-error :code code :message msg)))))
    


(defun get-error-code (&optional code)
  "Get a list of all error codes and messages.
If CODE is not null, get the message for a specific error code."
  (let (uri)
    (with-output-to-string (stream)
      (if code
          (format stream "~A~A?" *get-error-code* code)
          (format stream "~A?" *list-error-codes*))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-users (&key user-name (sort "username-asc") (count 10) (offset 0))
  "Get all Digg users.
If USER-NAME is specified, get informations for a specific user.
SORT specify how to sort returned users. It could be username-asc: A-Z,
or username-desc: Z-A (the default).
COUNT is the number of users to retrieve (default 10).
OFFSET in the complete user list."
  (let (uri)
    (with-output-to-string (stream)
      (if user-name
          (format stream "~A~A" *get-user* user-name)
          (format stream "~A" *list-users*))
      (format stream "?sort=~A" sort)
      (format stream "&count=~A" count)
      (format stream "&offset=~A&" offset)
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-user-friends (user-name
                         &key friend-name (sort "username-asc") (count 10) (offset 0))
  "Get named user's friends or friend' informations if FRIEND-NAME is specified.
The first user name is the befriender, the second is the befriended.
SORT specify how to sort returned users. It could be username-asc: A-Z,
or username-desc: Z-A (the default).
COUNT is the number of users to retrieve (default 10).
OFFSET in the complete user list."
  (let (uri)
    (with-output-to-string (stream)
      (if friend-name
          (format stream "~A~A"
                  (format nil *get-user-friend* user-name)
                  friend-name)
          (format stream "~A" (format nil *list-user-friends* user-name)))
      (format stream "?sort=~A" sort)
      (format stream "&count=~A" count)
      (format stream "&offset=~A&" offset)
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-user-fans (user-name
                      &key fan-name (sort "username-asc") (count 10) (offset 0))
  "Get named user's fans or fan' informations if FAN-NAME is specified.
The first user name is the befriender, the second is the befriended.
SORT specify how to sort returned users. It could be username-asc: A-Z,
or username-desc: Z-A (the default).
COUNT is the number of users to retrieve (default 10).
OFFSET in the complete user list."
  (let (uri)
    (with-output-to-string (stream)
      (if fan-name
          (format stream "~A~A"
                  (format nil *get-user-fan* user-name)
                  fan-name)
          (format stream "~A" (format nil *list-user-fans* user-name)))
      (format stream "?sort=~A" sort)
      (format stream "&count=~A" count)
      (format stream "&offset=~A&" offset)
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-topics (&optional topic)
  "Get a list of all topics or the specified TOPIC."
  (let (uri)
    (with-output-to-string (stream)
      (if topic
          (format stream "~A~A?" *get-topic* topic)
          (format stream "~A?" *list-topics*))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))



(defun list-stories (&key filter min-submit-date max-submit-date
                     min-promote-date max-promote-date                     
                     sort (count 10) (offset 0)
                     domain link)
  "Get Digg stories. FILTER could be :upcoming or :popular.
MIN-SUBMIT-DATE and MAX_SUBMIT_DATE for getting stories submitted within a given
time period.
MIN-PROMOTE-DATE and MAX-PROMOTE-DATE for getting stories promoted within a given
time period, implies.
SORT specify how to sort returned stories. It could be :promote-date-desc, :promote-date-asc,
:submit-date-desc or :submit-date-asc.
COUNT is the number of stories to retrieve The default is 10, and maximum 100.
OFFSET in complete story list.
DOMAIN is a partial domain of linked article.
LINK is the URL of linked article."
  (let (uri)
    (with-output-to-string (stream)
      (cond ((equal filter :popular)
             (format stream "~A?" *get-popular-stories*))
            ((equal filter :upcoming)
             (format stream "~A?" *get-upcoming-stories*))
            ((null filter)
             (format stream "~A?" *get-stories*))
            (t (error "Unknown filter request ~A" filter)))
      (when min-submit-date
        (format stream "&min_submit_date=~A" min-submit-date))
      (when max-submit-date
        (format stream "=&max_submit_date=~A" max-submit-date))
      (when min-promote-date
        (format stream "&min_promo_date=~A" min-promote-date))
      (when max-promote-date
        (format stream "=&max_promo_date=~A" max-promote-date))
      (when sort
        (cond ((equal sort :promote-date-desc)
               (format stream "&sort=promo_date_desc"))
              ((equal sort :promote-date-asc)
               (format stream "&sort=promo_date_asc"))
              ((equal sort :submit-date-desc)
               (format stream "&sort=submit_date_desc"))
              ((equal sort :submit-date-asc)
               (format stream "&sort=submit_date_asc"))
              (t (error "Unknown SORT parameter ~A" sort))))
      (when count
        (format stream "&count=~A" count))
      (when domain
        (format stream "&domain=~A" domain))
      (when link
        (format stream "&link=~A" link))
      (if offset
          (format stream "&offset=~A&" offset) 
          (format stream "&"))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun list-container-stories (container &key filter min-submit-date max_submit_date
                               min-promote-date max-promote-date
                               (sort "username-asc") (count 10) (offset 0)
                               domain link)
  "Get Digg stories for a given CONTAINER.
FILTER could be :upcoming or :popular.
MIN-SUBMIT-DATE and MAX_SUBMIT_DATE for getting stories submitted within a given
time period.
MIN-PROMOTE-DATE and MAX-PROMOTE-DATE for getting stories promoted within a given
time period, implies.
SORT specify how to sort returned stories. It could be :promote-date-desc, :promote-date-asc,
:submit-date-desc or :submit-date-asc.
COUNT is the number of stories to retrieve The default is 10, and maximum 100.
OFFSET in complete story list.
DOMAIN is a partial domain of linked article.
LINK is the URL of linked article."
  (let (uri)
    (with-output-to-string (stream)
      (cond ((equal filter :popular)
             (format stream "~A?"
                     (format nil *get-popular-container-stories* container)))
            ((equal filter :upcoming)
             (format stream "~A?"
                     (format nil *get-upcoming-container-stories* container)))
            ((null filter)
             (format stream "~A~A?" *get-container-stories* container))
            (t (error "Unknown filter request ~A" filter)))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun list-topic-stories (topic &key filter min-submit-date max_submit_date
                           min-promote-date max-promote-date
                           (sort "username-asc") (count 10) (offset 0)
                           domain link)                           
  "Get Digg stories for a given TOPIC.
FILTER could be :upcoming or :popular.
MIN-SUBMIT-DATE and MAX_SUBMIT_DATE for getting stories submitted within a given
time period.
MIN-PROMOTE-DATE and MAX-PROMOTE-DATE for getting stories promoted within a given
time period, implies.
SORT specify how to sort returned stories. It could be :promote-date-desc, :promote-date-asc,
:submit-date-desc or :submit-date-asc.
COUNT is the number of stories to retrieve The default is 10, and maximum 100.
OFFSET in complete story list.
DOMAIN is a partial domain of linked article.
LINK is the URL of linked article."
  (let (uri)
    (with-output-to-string (stream)
      (cond ((equal filter :popular)
             (format stream "~A?"
                     (format nil *get-popular-topic-stories* topic)))
            ((equal filter :upcoming)
             (format stream "~A?"
                     (format nil *get-upcoming-topic-stories* topic)))
            ((null filter)
             (format stream "~A~A?" *get-topic-stories* topic))
            (t (error "Unknown filter request ~A" filter)))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun list-user-stories (username &key min-submit-date max_submit_date
                           min-promote-date max-promote-date
                          (sort "username-asc") (count 10) (offset 0)
                          domain link)                          
  "Get Digg stories submitted by given user.
MIN-SUBMIT-DATE and MAX_SUBMIT_DATE for getting stories submitted within a given
time period.
MIN-PROMOTE-DATE and MAX-PROMOTE-DATE for getting stories promoted within a given
time period, implies.
SORT specify how to sort returned stories. It could be :promote-date-desc, :promote-date-asc,
:submit-date-desc or :submit-date-asc.
COUNT is the number of stories to retrieve The default is 10, and maximum 100.
OFFSET in complete story list.
DOMAIN is a partial domain of linked article.
LINK is the URL of linked article."
  (let (uri)
    (with-output-to-string (stream)
      (format stream "~A?" (format nil *get-user-stories* username))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-story (&key ids title min-submit-date max-submit-date
                  min-promote-date max-promote-date
                  (sort "username-asc") (count 10) (offset 0)
                  domain link)
  "Get Digg stories. IDS is a list of Identifiant.
MIN-SUBMIT-DATE and MAX_SUBMIT_DATE for getting stories submitted within a given
time period.
MIN-PROMOTE-DATE and MAX-PROMOTE-DATE for getting stories promoted within a given
time period, implies.
SORT specify how to sort returned stories. It could be :promote-date-desc, :promote-date-asc,
:submit-date-desc or :submit-date-asc.
COUNT is the number of stories to retrieve The default is 10, and maximum 100.
OFFSET in complete story list.
DOMAIN is a partial domain of linked article.
LINK is the URL of linked article."
  (unless (or ids title)
    (error "Must defined IDS or TITLE."))
  (let (uri)
    (with-output-to-string (stream)
      (if ids
          (if (= 1 (length ids))
              (format stream "~A~A?" *get-story* (first ids))
              (format stream "~A/~{~A~^,~}?" *get-stories* ids))
          (format stream "~A~A?" *get-story* title))
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-diggs (&key filter ids min-date max-date (sort :date-desc)
                  (count 10) (offset 0))
  "Get diggs.
FILTER could be :upcoming or :popular.
IDS is a list of stories with the given ids.
MIN-DATE and MAX-DATE specify a time period.
SORT is how to sort returned events. It could be :date-desc (default) or :date-asc.
COUNT is the number of events to retrieve.
OFFSET in complete events list."
  (let (uri)
    (with-output-to-string (stream)
      (cond ((equal filter :popular)
             (format stream "~A" *popular-diggs*))
            ((equal filter :upcoming)
             (format stream "~A" *upcoming-diggs*))
            ((null filter)
             (cond ((> (length ids) 1)
                    (format stream *stories-diggs* ids))
                   ((= (length ids) 1)
                    (format stream *story-diggs* ids))
                   (t
                    (format stream "~A" *comments*)))))
      (format stream "?")
      (when min-date
        (format stream "&min_date=~A" min-date))
      (when max-date
        (format stream "&max_date=~A" max-date))
      (when sort
        (cond ((equal :date-desc sort)
               (format stream "&sort=date-desc"))
              ((equal :date-asc sort)
               (format stream "&sort=date-asc"))
              (t (error "Unknown sort ~A" sort))))
      (when count
        (format stream "&count=~A" count))
      (when offset
        (format stream "&offset=~A" offset))
      (format stream "&")
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-comments (&key filter ids min-date max-date (sort :date-desc)
                     (count 10) (offset 0))
  "Get comments.
FILTER could be :upcoming or :popular.
IDS is a list of stories with the given ids.
MIN-DATE and MAX-DATE specify a time period.
SORT is how to sort returned events. It could be :date-desc (default) or :date-asc.
COUNT is the number of events to retrieve.
OFFSET in complete events list."
  (let (uri)
    (with-output-to-string (stream)
      (cond ((equal filter :popular)
             (format stream "~A" *popular-comments*))
            ((equal filter :upcoming)
             (format stream "~A" *upcoming-comments*))
            ((null filter)
             (cond ((> (length ids) 1)
                    (format stream *stories-comments* ids))
                   ((= (length ids) 1)
                    (format stream *story-comments* ids))
                   (t
                    (format stream "~A" *comments*)))))
      (format stream "?")
      (when min-date
        (format stream "&min_date=~A" min-date))
      (when max-date
        (format stream "&max_date=~A" max-date))
      (when sort
        (cond ((equal :date-desc sort)
               (format stream "&sort=date-desc"))
              ((equal :date-asc sort)
               (format stream "&sort=date-asc"))
              (t (error "Unknown sort ~A" sort))))
      (when count
        (format stream "&count=~A" count))
      (when offset
        (format stream "&offset=~A" offset))
      (format stream "&")
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-story-events (story-id event &key min-date max-date (sort :date-desc)
                         (count 10) (offset 0))
  "Get all diggs or comments for a given story STORY-ID.
EVENT could be :diggs or :comments.
MIN-DATE and MAX-DATE specify a time period.
SORT is how to sort returned events. It could be :date-desc (default) or :date-asc.
COUNT is the number of events to retrieve.
OFFSET in complete events list."
  (let (uri)
    (with-output-to-string (stream)
      (cond ((equal :diggs event)
             (format stream *story-diggs* story-id))
            ((equal :comments event)
             (format stream *story-comments* story-id))
            (t (error "Unknown event ~A" event)))
      (when min-date
        (format stream "&min_date=~A" min-date))
      (when max-date
        (format stream "&max_date=~A" max-date))
      (when sort
        (cond ((equal :date-desc sort)
               (format stream "&sort=date-desc"))
              ((equal :date-asc sort)
               (format stream "&sort=date-asc"))
              (t (error "Unknown sort ~A" sort))))
      (when count
        (format stream "&count=~A" count))
      (when offset
        (format stream "&offset=~A" offset))
      (format stream "?")
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


(defun get-users-events (users event &key min-date max-date (sort :date-desc)
                        (count 10) (offset 0))
  "Get several users' diggs or comments for all stories.
EVENT could be :diggs or :comments.
MIN-DATE and MAX-DATE specify a time period.
SORT is how to sort returned events. It could be :date-desc (default) or :date-asc.
COUNT is the number of events to retrieve.
OFFSET in complete events list."
  (let (uri)
    (with-output-to-string (stream)
      (cond ((equal :diggs event)
             (format stream *users-diggs* users))
            ((equal :comments event)
             (format stream *user-comments* users))
            (t (error "Unknown event ~A" event)))
      (when min-date
        (format stream "&min_date=~A" min-date))
      (when max-date
        (format stream "&max_date=~A" max-date))
      (when sort
        (cond ((equal :date-desc sort)
               (format stream "&sort=date-desc"))
              ((equal :date-asc sort)
               (format stream "&sort=date-asc"))
              (t (error "Unknown sort ~A" sort))))
      (when count
        (format stream "&count=~A" count))
      (when offset
        (format stream "&offset=~A" offset))
      (format stream "?")
      (setf uri (get-output-stream-string stream)))
    (with-digg-request (request) uri
      (digg-perform request))))


